﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PseudoInterpreter.Recursive
{
    public class StackMachine
    {
        private sealed class Status
        {
            public IEnumerator<ResultBase> Frame;
            public ResultBase ResultStorage;
        }

        private Stack<Status> stack = new Stack<Status>();
        private Status CurrentStatus = null;

        public Result<T> Call<T, R>(IEnumerable<Result<R>> method, Result<R> resultStorage)
        {
            stack.Push(new Status() { Frame = method.GetEnumerator(), ResultStorage = resultStorage });
            return new Result<T>() { Dummy = true };
        }
        public Result<T> Call<T, R>(IEnumerable<Result<R>> method)
        {
            Result<R> resultStorage = new Result<R>();
            stack.Push(new Status() { Frame = method.GetEnumerator(), ResultStorage = resultStorage });
            return new Result<T>() { Dummy = true };
        }
        public Result<T> Call<T>(IEnumerable<Result> method)
        {
            Result voidStorage = new Result();
            stack.Push(new Status() { Frame = method.GetEnumerator(), ResultStorage = voidStorage });
            return new Result<T>() { Dummy = true };
        }
        public Result Call<R>(IEnumerable<Result<R>> method, Result<R> resultStorage)
        {
            stack.Push(new Status() { Frame = method.GetEnumerator(), ResultStorage = resultStorage });
            return new Result() { Dummy = true };
        }
        public Result Call(IEnumerable<Result> method)
        {
            Result voidStorage = new Result();
            stack.Push(new Status() { Frame = method.GetEnumerator(), ResultStorage = voidStorage });
            return new Result() { Dummy = true };
        }
        public Result<T> Return<T>(IEnumerable<Result<T>> method)
        {
            CurrentStatus.Frame = method.GetEnumerator();
            return new Result<T>() { Dummy = true };
        }
        public Result<T> Return<T, R>(IEnumerable<Result<R>> method) where R:T {
            CurrentStatus.Frame = method.GetEnumerator();
            return new Result<T>() { Dummy = true };
        }
        public void CallImmediate(IEnumerable<Result> method)
        {
            if (stack.Count == 0) { stack.Push(null); }//dummy value
            Call(method);
            Run();
        
        }
        public T CallImmediate<T>(IEnumerable<Result<T>> method)
        {
            if (stack.Count == 0) { stack.Push(null); }//dummy value

            Result<T> result = new Result<T>();
            Call<T, T>(method, result);
            Run();
            return (T)result.Value;
        }
        private void Run() {
            Status mainStatus = stack.Peek();

            while (true)
            {
                CurrentStatus = stack.Peek();
                IEnumerator<ResultBase> currentFrame = CurrentStatus.Frame;
                if (currentFrame.MoveNext())
                {
                    ResultBase r = currentFrame.Current;
                    if (r == null)
                    {
                        CurrentStatus.ResultStorage.SetDefault();
                        stack.Pop();
                        if (CurrentStatus == mainStatus)
                        {
                            CurrentStatus = stack.Peek();
                            break;
                        }
                    }
                    else if (!r.Dummy)
                    {
                        CurrentStatus.ResultStorage.SetValue(r.GetValue());
                        stack.Pop();
                        if (CurrentStatus == mainStatus)
                        {
                            CurrentStatus = stack.Peek();
                            break;
                        }
                    }
                }
                else
                {
                    stack.Pop();
                    if (CurrentStatus == mainStatus)
                    {
                        CurrentStatus = stack.Peek();
                        break;
                    }
                }
            }
        }
    }
}
