﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using PseudoInterpreter.Interpreter;
using PseudoInterpreter.Ast;
using PseudoInterpreter.Recursive;

namespace PseudoInterpreter.UI
{
    /// <summary>
    /// Logica di interazione per InterpreterControl.xaml
    /// </summary>
    public partial class InterpreterControl : UserControl
    {
        public InterpreterControl()
        {
            InitializeComponent();
            Interpret();
        }
        private void Interpret() {
            Action callable = () =>
            {
                try
                {
                    AstParser parser = new AstParser(new System.IO.StringReader(CodeProvider.Code));
                    AstInterpreter ai = new AstInterpreter(parser.ParseTree());
                    InterpreterEnvironment env = new InterpreterEnvironment(
                        (x) => Dispatcher.Invoke(() => ConsoleArea.Text += x)
                    );
                    ai.Expone("native", (ObjectProvider)new Dictionary<string, ReferenceValue>() {
                        {"time", (NativeMethodProvider)env.Time },
                        {"dump", (NativeMethodProvider)env.Dump }
                    });
                    ai.Expone("min", (NativeMethodProvider)env.Min);
                    ai.Expone("max", (NativeMethodProvider)env.Max);
                    ai.SetPrintBinding(env.Print);
                    ai.Interpret();
                }
                catch (Exception ex) {
                    Dispatcher.Invoke(() =>
                    {
                        ConsoleArea.Text = ex.Message;
                        ConsoleArea.Foreground = new SolidColorBrush(Color.FromRgb(0xdc, 0x32, 0x2f));
                    });
                }
            };
            callable.BeginInvoke(null, null);
        }
    }
}
