﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using PseudoInterpreter.Tokenizer;

namespace PseudoInterpreter.UI
{
    /// <summary>
    /// Logica di interazione per TokenContro.xaml
    /// </summary>
    public partial class TokenControl : UserControl
    {
        IEnumerable<Token> Tokens;
        public TokenControl()
        {
            InitializeComponent();
            var tokenizer = new Tokenizer.Tokenizer(new System.IO.StringReader(CodeProvider.Code));
            try
            {
                Tokens = tokenizer.Tokenize();
                PrepareDocument();
            }
            catch (Exception ex) {
                PrepareErrorDocument(ex.Message);
            }
        }
        private static SolidColorBrush Green = new SolidColorBrush(Color.FromRgb(0x85, 0x99, 0x00));
        private static SolidColorBrush Cyan = new SolidColorBrush(Color.FromRgb(0x2a, 0xa1, 0x98));
        private static SolidColorBrush Blue = new SolidColorBrush(Color.FromRgb(0x26, 0x8b, 0xd2));
        private static SolidColorBrush Violet = new SolidColorBrush(Color.FromRgb(0x6c, 0x71, 0xc4));
        private static SolidColorBrush Magenta = new SolidColorBrush(Color.FromRgb(0xd3, 0x36, 0x82));
        private static SolidColorBrush Red = new SolidColorBrush(Color.FromRgb(0xdc, 0x32, 0x2f));
        private static SolidColorBrush Orange = new SolidColorBrush(Color.FromRgb(0xcb, 0x4b, 0x16));
        private static SolidColorBrush Yellow = new SolidColorBrush(Color.FromRgb(0xb5, 0x89, 0x00));
        private static SolidColorBrush White = new SolidColorBrush(Colors.White);


        private void PrepareErrorDocument(string error)
        {
            Paragraph p = new Paragraph();
            p.Margin = new Thickness(0);
            FlowDocument document = new FlowDocument();
            Run r = new Run();
            r.Text = error;
            r.Foreground = Red;
            p.Inlines.Add(r);
            document.Blocks.Add(p);
            CodeArea.Document = document;
        }

        private void PrepareDocument() {
            Paragraph p = new Paragraph();
            p.Margin = new Thickness(0);
            FlowDocument document = new FlowDocument();

            foreach (Token token in Tokens) {
                switch (token.TokenType) {
                    case TokenType.EndInput:
                    case TokenType.EndBlock:
                    case TokenType.StartBlock:
                        if (p.Inlines.Count != 0) {
                            p = new Paragraph();
                            p.Margin = new Thickness(0);
                        }
                        p.Inlines.AddRange(FormatToken(token));
                        document.Blocks.Add(p);
                        p = new Paragraph();
                        p.Margin = new Thickness(0);
                        break;
                    case TokenType.EndStatement:
                        if (p.Inlines.Count == 0) { document.Blocks.Add(p); }
                        p.Inlines.AddRange(FormatToken(token));
                        p = new Paragraph();
                        p.Margin = new Thickness(0);
                        break;
                    default:
                        if (p.Inlines.Count == 0) { document.Blocks.Add(p); }
                        p.Inlines.AddRange(FormatToken(token));
                        break;
                }
            }
            CodeArea.Document = document;
        }
        private IEnumerable<Inline> FormatToken(Token token){
            List<Inline> result = new List<Inline>();
            result.Add(new Run() { Text = "[", Foreground = new SolidColorBrush(Colors.White) });
            Run middle = new Run();
            result.Add(middle);
            switch (token.TokenType) {
                case TokenType.Else:
                case TokenType.If:
                case TokenType.For:
                case TokenType.Return:
                case TokenType.To:
                case TokenType.While:
                case TokenType.Swap:
                case TokenType.Downto:
                case TokenType.Repeat:
                case TokenType.Until:
                case TokenType.Print:
                case TokenType.Each:
                case TokenType.Then:
                    middle.Text = token.TokenType.ToString();
                    middle.Foreground = Yellow;
                    break;

                case TokenType.LogicalAnd:
                case TokenType.LogicalOr:
                case TokenType.LogicalNot:
                case TokenType.EndGroup:
                case TokenType.StartGroup:
                case TokenType.StartIndexer:
                case TokenType.EndIndexer:
                case TokenType.MemberAccessor:
                case TokenType.StartFloor:
                case TokenType.EndFloor:
                case TokenType.StartCeiling:
                case TokenType.EndCeiling:
                case TokenType.StartSet:
                case TokenType.EndSet:
                    middle.Text = token.TokenType.ToString();
                    middle.Foreground = Blue;
                    break;

                case TokenType.Div:
                case TokenType.Minus:
                case TokenType.Mult:
                case TokenType.Plus:
                case TokenType.Remainder:
                case TokenType.Range:
                case TokenType.Assignment:
                case TokenType.Different:
                case TokenType.Greater:
                case TokenType.GreaterEqual:
                case TokenType.Less:
                case TokenType.LessEqual:
                case TokenType.Equal:
                case TokenType.DoubleArrow:
                case TokenType.ElementOf:
                case TokenType.NotElementOf:
                case TokenType.Union:
                case TokenType.Intersection:
                case TokenType.SetMinus:
                case TokenType.SubsetOf:
                case TokenType.SupersetOf:
                case TokenType.SupersetOfEqual:
                case TokenType.SubsetOfEqual:
                    middle.Text = token.TokenType.ToString();
                    middle.Foreground = Red;
                    break;

                case TokenType.ConstantDouble:
                    middle.Text = "Double "+((FloatToken)token).Value;
                    middle.Foreground = Green;
                    break;
                case TokenType.ConstantInteger:
                    middle.Text = "Integer " + ((IntegerToken)token).Value;
                    middle.Foreground = Green;
                    break;
                case TokenType.Identifier:
                    middle.Text = token.TokenType.ToString() + " " + ((IdentifierToken)token).Value;
                    middle.Foreground = Green;
                    break;
                case TokenType.EndArgument:
                case TokenType.EndBlock:
                case TokenType.EndInput:
                case TokenType.EndStatement:
                case TokenType.StartBlock:
                    middle.Foreground = White;
                    middle.Text = token.TokenType.ToString();
                    break;

                case TokenType.Infinity:
                case TokenType.Nil:
                case TokenType.EmptySet:
                    middle.Text = token.TokenType.ToString();
                    middle.Foreground = Orange;
                    break;

            }
            result.Add(new Run() { Text = "] ", Foreground = new SolidColorBrush(Colors.White) });
            return result;
        }

        private void CodeArea_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            CodeArea.Document.PageWidth = CodeArea.ActualWidth;
        }
    }
}
