﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PseudoInterpreter.Tokenizer;

namespace PseudoInterpreter.Ast
{
    internal static class AstParserHelper
    {
        internal static BinaryOperator? GetBinaryOperator(TokenType tokenType)
        {
            switch (tokenType)
            {
                case TokenType.Assignment: return BinaryOperator.Assign;
                case TokenType.Different: return BinaryOperator.Different;
                case TokenType.Div: return BinaryOperator.Div;
                case TokenType.Equal: return BinaryOperator.Equal;
                case TokenType.Greater: return BinaryOperator.Greater;
                case TokenType.GreaterEqual: return BinaryOperator.GreaterEqual;
                case TokenType.Less: return BinaryOperator.Less;
                case TokenType.LessEqual: return BinaryOperator.LessEqual;
                case TokenType.LogicalAnd: return BinaryOperator.LogicalAnd;
                case TokenType.LogicalOr: return BinaryOperator.LogicalOr;
                case TokenType.Minus: return BinaryOperator.Minus;
                case TokenType.Mult: return BinaryOperator.Mult;
                case TokenType.Plus: return BinaryOperator.Plus;
                case TokenType.Remainder: return BinaryOperator.Remainder;
                case TokenType.Union: return BinaryOperator.Union;
                case TokenType.Intersection: return BinaryOperator.Intersection;
                case TokenType.ElementOf: return BinaryOperator.ElementOf;
                case TokenType.NotElementOf: return BinaryOperator.NotElementOf;
                case TokenType.SetMinus: return BinaryOperator.SetMinus;
                case TokenType.SupersetOf: return BinaryOperator.SupersetOf;
                case TokenType.SubsetOf: return BinaryOperator.SubsetOf;
                case TokenType.SupersetOfEqual: return BinaryOperator.SupersetOfEqual;
                case TokenType.SubsetOfEqual: return BinaryOperator.SubsetOfEqual;
                default: return null;
            }
        }
        internal static int GetOperatorPriority(BinaryOperator op)
        {
            switch (op)
            {
                case BinaryOperator.Plus:
                case BinaryOperator.Minus:
                case BinaryOperator.SetMinus:
                    return 900;
                case BinaryOperator.Mult:
                case BinaryOperator.Div:
                case BinaryOperator.Remainder:
                case BinaryOperator.Intersection:
                    return 1000;
                case BinaryOperator.Union:
                    return 950;
                case BinaryOperator.LogicalAnd:
                    return 750;
                case BinaryOperator.LogicalOr:
                    return 700;
                case BinaryOperator.Assign:
                    return 600;
                case BinaryOperator.Less:
                case BinaryOperator.LessEqual:
                case BinaryOperator.Greater:
                case BinaryOperator.GreaterEqual:
                case BinaryOperator.Equal:
                case BinaryOperator.Different:
                case BinaryOperator.SubsetOf:
                case BinaryOperator.SubsetOfEqual:
                case BinaryOperator.SupersetOf:
                case BinaryOperator.SupersetOfEqual:
                case BinaryOperator.ElementOf:
                case BinaryOperator.NotElementOf:
                    return 800;
            }
            throw new Exception("BinaryOperator was changed");
        }
    }
}
