﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PseudoInterpreter.Recursive;

namespace PseudoInterpreter.Interpreter
{
    [System.ComponentModel.ImmutableObject(true)]
    public sealed class NativeMethodProvider : Provider, ICallProvider
    {
        public NativeMethodProvider(Func<AstInterpreter, ReferenceValue[], IEnumerable<Result<Provider>>> method)
            : base()
        {
            this.Method = method;
        }

        internal readonly Func<AstInterpreter, ReferenceValue[], IEnumerable<Result<Provider>>> Method;
        public override IEnumerable<Result<Provider>> Negate(AstInterpreter ai, ReferenceValue thisReference) { yield return Constants.Zero; }
        public override bool IsTrue(ReferenceValue thisReference) { return true; }

        #region ICallProvider
        public IEnumerable<Result<Provider>> Call(AstInterpreter ai, ReferenceValue thisReference, ReferenceValue[] arguments)
        {
            yield return ai.Return(this.Method(ai, arguments));
        }
        #endregion

        public static implicit operator NativeMethodProvider(Func<AstInterpreter, ReferenceValue[], IEnumerable<Result<Provider>>> method)
        {
            return new NativeMethodProvider(method);
        }
        public static implicit operator NativeMethodProvider(Func<ReferenceValue[], Provider> method)
        {

            return new NativeMethodProvider(
                (x, y) => {
                    return new List<Result<Provider>>() { method(y) };
                }
           );
        }
        public static implicit operator NativeMethodProvider(Func<AstInterpreter, Provider> method)
        {
            return new NativeMethodProvider((x, y) =>
            {
                if (y.Length != 0) { return new List<Result<Provider>>() { Constants.Nil }; }
                return new List<Result<Provider>>() { method(x) };
            });
        }
        public static implicit operator NativeMethodProvider(Func<AstInterpreter, IEnumerable<Result<Provider>>> method)
        {
            return new NativeMethodProvider((x, y) =>
            {
                if (y.Length != 0) { return new List<Result<Provider>>() { Constants.Nil }; }
                return method(x);
            });
        }
        public static implicit operator NativeMethodProvider(Func<Provider> method)
        {
            return new NativeMethodProvider((x, y) =>
            {
                if (y.Length != 0) { return new List<Result<Provider>>() { Constants.Nil }; }
                return new List<Result<Provider>>() { method() };
            });
        }
    }
}
