﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PseudoInterpreter.Recursive;

namespace PseudoInterpreter.Interpreter
{
    public sealed class ObjectProvider : Provider, IObjectProvider
    {
        public readonly Dictionary<string, ReferenceValue> Value;
        public ObjectProvider() { Value = new Dictionary<string, ReferenceValue>(); }


        #region Provider
        public override IEnumerable<Result<Provider>> Negate(AstInterpreter ai, ReferenceValue thisReference)
        {
            yield return Value.Count == 0 ? Constants.One : Constants.Zero;
        }
        public override bool IsTrue(ReferenceValue thisReference)
        {
            return Value.Count > 0;
        }
        #endregion

        #region IObjectProvider
        public ReferenceValue GetProperty(ReferenceValue thisReference, string name)
        {
            if (!Value.ContainsKey(name))
            {
#if TYPE_UNDEFINED
                Value[name] = Constants.Undefined;
#else
                Value[name] = Constants.Nil;
#endif
            }
            return Value[name];
        }
        #endregion


        public static implicit operator ObjectProvider(Dictionary<string, ReferenceValue> data)
        {
            ObjectProvider provider = new ObjectProvider();
            foreach (var kvp in data)
            {
                provider.Value.Add(kvp.Key, kvp.Value);
            }
            return provider;
        }
        public static implicit operator ObjectProvider(List<KeyValuePair<string, ReferenceValue>> data)
        {
            ObjectProvider provider = new ObjectProvider();
            foreach (var kvp in data)
            {
                provider.Value.Add(kvp.Key, kvp.Value);
            }
            return provider;
        }
        public static implicit operator ObjectProvider(KeyValuePair<string, ReferenceValue>[] data)
        {
            ObjectProvider provider = new ObjectProvider();
            foreach (var kvp in data)
            {
                provider.Value.Add(kvp.Key, kvp.Value);
            }
            return provider;
        }

    }
}
