﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PseudoInterpreter.Interpreter
{
    public sealed class SparseArray : IEnumerable<KeyValuePair<int, ReferenceValue>>
    {
        private Dictionary<int, ReferenceValue> values;
        private int length;
        private ReferenceValue lengthReference;
        public int Length
        {
            get { return length; }
            set
            {
                if (value < 0) { throw new InvalidOperationException("length cannot be negative"); }

                if (value == length) { return; }
                for (; length > value; --length)
                {
                    this.values.Remove(length);
                }
                length = value;
                lengthReference.SetValue(new IntegerConstantProvider(length), false);
            }
        }


        public IEnumerator<KeyValuePair<int, ReferenceValue>> GetEnumerator()
        {
            return values.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        public ReferenceValue this[int index]
        {
            get { return values[index]; }
            set {
                if (index > length) { Length = index; }
                values[index] = value;
            }
        }

        internal SparseArray(Dictionary<int, ReferenceValue> values, ReferenceValue lengthReference, int length) {
            this.lengthReference = lengthReference;
            this.length = -1;
            this.values = values;
            this.Length = length;
        }

        public void Add(int key, ReferenceValue value) {
            values.Add(key, value);
        }
        public bool ContainsKey(int key) { return values.ContainsKey(key); }
    }
}
