﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;

namespace PseudoInterpreter.Tokenizer
{
    internal sealed class TokenizerReader
    {
        private TextReader Reader;
        /// <summary>
        /// Linea del carattere letto
        /// </summary>
        public int Line { get; private set; }
        /// <summary>
        /// Colonna del carattere letto
        /// </summary>
        public int Column { get; private set; }

        public TokenizerReader(TextReader reader) {
            this.Reader = reader;
            this.Line = 0;
            this.Column = 0;
        }
        /// <summary>
        /// Restituisce il successivo carattere disponibile e avanza di una posizione
        /// </summary>
        /// <returns>-1 se EOI, il codice del carattere</returns>
        public int Read() {
            int ch = this.Reader.Read();
            if (ch == '\n') {
                ++this.Line;
                this.Column = 0;
            }
            else if (ch != -1) {
                ++this.Column;
            }
            return ch;
        }

        /// <summary>
        /// Restituisce il successivo carattere disponibile senza leggerlo effettivamente
        /// </summary>
        /// <returns>-1 se EOI, il codice del carattere</returns>
        public int Peek() {
            return this.Reader.Peek();
        }
    }
}
